%
%
%   UndStdLeakyMode
%   Understand leaky modes: Slab waveguide revisited
%
%   Copyright (2008)
%   Computational Photonics Laboratory
%   University of Maryland Baltimore County
%   
%   By Jonathan Hu
%



function AbsorbingLayer(MaxLength,dim,WidthAbsorbingLayer,sPP,baratio,NumEigs,FigNumber)


% This function generate figures for a W-type waveguide.
%
% INPUT:
% 
% MaxLength  -----    equal to L, see Fig. 23 in tutorial paper
% dim ----- number of decomposition series
% WidthAbsorbingLayer ------ width of absorbing layer
% sPP ------ s'
% baratio ------ b=baratio*a
% NumEigs ------ number of eigen values
% FigNumber ------ Figure index
%



tStart=clock;

%close all;
sq=inline('x.*x','x');
lambda=1e-6; % wavelength

n1=1.45; % refractive index of the core
n2=n1*0.96; % refractive index of the cladding
E1=sq(n1); % epsilon, 
E2=sq(n2); % epsilon, 



wavenumber=2*pi/lambda;
a=sqrt(1/wavenumber^2/(n1^2-n2^2)); % half of the core width


b=baratio*a;

xmin=-(MaxLength+WidthAbsorbingLayer); % -B
xmax=MaxLength+WidthAbsorbingLayer; % B
DeltaX=(xmax-xmin)/(dim);
f=1/(DeltaX*wavenumber)^2;

prm0=ones(dim,1);
WidthAbsorbingLayer=round(WidthAbsorbingLayer/DeltaX)*DeltaX;
for ii=1:length(prm0)
    
    if(abs(xmin+(ii-1)*DeltaX)<a)
       prm0(ii)=E1;
    else
        if(abs(xmin+(ii-1)*DeltaX)<b)
            prm0(ii)=E2;
        else
            prm0(ii)=E1;
        end;
    end;
    if abs(xmin+(ii-1)*DeltaX)>xmax-WidthAbsorbingLayer
        prm0(ii)=prm0(ii)*(1-i*((abs(xmin+(ii-1)*DeltaX)-(xmax-WidthAbsorbingLayer))/WidthAbsorbingLayer).^2*sPP);
    end;
end;

% form the matrix for eigen value equation
RefTarget=0;
L=sparse([2:dim 1:dim 1:dim-1]',[1:dim-1 1:dim 2:dim]', ...
    [f*ones(dim-1,1); (-2*f-sq(RefTarget))*ones(dim,1)+prm0; f*ones(dim-1,1)],dim,dim);


tol=1e-22;
opts.tol = tol;
opts.disp=1;

    runtime = etime(clock,tStart);
    fprintf('run time = %fs or %fm or %fh\n',runtime,runtime/60,runtime/3600);




% calculate left and right eigen vector and value. 
[evecR, evalR]=eig(full(L));
eff_nR=sqrt(diag(evalR));
[C I]=sort(real(eff_nR));
eff_nR=eff_nR(I);
evecR=evecR(:,I);

[evecL, evalL]=eig(full(L.'));
eff_nL=sqrt(diag(evalL)+sq(RefTarget));
[C I]=sort(real(eff_nL));
evecL=evecL(:,I);


% Calculate coefficient D
CoeffD=diag(evecL.'*evecR);

InputU=exp(-1*((xmin+((1:dim)'-1)*DeltaX).^2)/2/a/a);

CoeffC=evecL.'*InputU;

if FigNumber==27
    
    %figure;plot(1:dim,[real(CoeffC) imag(CoeffC)]);
    %figure;plot(1:dim,[real(eff_nR) abs(imag(eff_nR))]);
  
  
    figure;[AX,H1,H2] = plotyy(1:dim,abs(imag(eff_nR)),...
        1:dim,real(eff_nR),'semilogy','plot');
    set(get(AX(1),'Ylabel'),'String','Im(n_{eff})'); 
    set(get(AX(2),'Ylabel'),'String','Re(n_{eff})');
    set(get(AX(1),'Xlabel'),'String','Mode index');
    set(H1,'LineWidth',1,'color','r');
    set(H2,'LineWidth',1,'Color','b');
    ylim(AX(1),[1e-5 10]);set(AX(1),'ytickMode','auto');
    ylim(AX(2),[0 2]);set(AX(2),'ytickMode','auto');
end;


eff_nR=real(eff_nR)-i*abs(imag(eff_nR));


if FigNumber==28
    AbsorbingLayerPropagation(dim,a,b,MaxLength,WidthAbsorbingLayer,DeltaX,CoeffC,CoeffD,evecR,eff_nR,wavenumber);
end;

    
if FigNumber==29
    Distance=linspace(1,50000,2000)*1e-6;
    ExpandInputUCenter=zeros(length(Distance),1);
    for Zii=1:length(Distance)
        ExpandInputUCenter(Zii)=evecR(end/2+1,:) ...
            *(CoeffC./CoeffD.*exp(-i*eff_nR*wavenumber*Distance(Zii)));
    end;
    figure;semilogy(Distance/lambda, (abs(ExpandInputUCenter)).^2,'b','linewidth',2);
    for Zii=1:length(Distance)
         ExpandInputUCenter(Zii)=evecR(end/2+1,476) ...
             *(CoeffC(476)/CoeffD(476)*exp(-i*eff_nR(476)*wavenumber*Distance(Zii)));
    end;        
    hold on;semilogy(Distance/lambda, (abs(ExpandInputUCenter)).^2,'r--','linewidth',2); 
    ylim([1e-30 1]);
    ylabel('I_{norm}');xlabel('z/\lambda');
end;








    
    
if FigNumber==30
    AbsorbingLayerDecomp(InputU,dim,a,b,MaxLength,WidthAbsorbingLayer,DeltaX,CoeffC,CoeffD,evecR,eff_nR,wavenumber)
  
end;




    runtime = etime(clock,tStart);
    fprintf('run time = %fs or %fm or %fh\n',runtime,runtime/60,runtime/3600);
